#!/usr/local/bin/python
#
# $Id: //projects/csbuddy/cslog.py#8 $ $Date: 2003/03/16 $

"""
Counter-Strike log parsing system.
"""

__package__ = 'csbuddy'


import re
import time
import types


class Parser:
    
    """The basic CS log parser.  Given a file object, transform lines into
    events with a date stamp (the 'now' attribute of the parser)."""
    
    EVENT_RES = {'logStart': re.compile('^Log file started'),
                 'logEnd': re.compile('^Log file closed$'),
                 'loadingMap': re.compile('^Loading map "([^\"]*)"$'),
                 'startedMap': re.compile('^Started map "([^\"]*)" \(CRC "([^\"]*)"\)$'),
                 'serverSay': re.compile('^Server say "([^\"]*)"$'),
                 'serverShutdown': re.compile('^Server shutdown$'),
                 'rcon': re.compile('^Rcon: "rcon (\d+) "([^\"]*)" ([^\"]*)" from "([^\"]*)"$'),
                 'badRcon': re.compile('^Bad Rcon: "rcon (\d+) "([^\"]*)" ([^\"]*)" from "([^\"]*)"$'),
                 'cvarStart': re.compile('^Server cvars start$'),
                 'cvarEnd': re.compile('^Server cvars end$'),
                 'cvar': re.compile('^Server cvar "([^\"]*)" = "([^\"]*)"$'),
                 'trigger': re.compile('^(.*) triggered "([^\"]*)"'),
                 'connect': re.compile('^"([^\"]*)" connected, address "([^\"]*)"$'),
                 'validated': re.compile('^"([^\"]*)" STEAM USERID validated$'),
                 'disconnect': re.compile('^"([^\"]*)" disconnected$'),
                 'enter': re.compile('^"([^\"]*)" entered the game$'),
                 'join': re.compile('^"([^\"]*)" joined team "([^\"]*)"( \(auto\))?$'),
                 'nameChange': re.compile('^"([^\"]*)" changed name to "([^\"]*)"$'),
                 'attack1.3': re.compile('^"([^\"]*)" attacked "([^\"]*)" with "([^\"]*)" \(damage "health:([^\"]*)"\) \(damage "armor:([^\"]*)"\) \(health "([^\"]*)"\) \(armor "([^\"]*)"\)$'),
                 'attack1.4': re.compile('^"([^\"]*)" attacked "([^\"]*)" with "([^\"]*)" \(damage "([^\"]*)"\) \(damage_armor "([^\"]*)"\) \(health "([^\"]*)"\) \(armor "([^\"]*)"\)$'),
                 'kill': re.compile('^"([^\"]*)" killed "([^\"]*)" with "([^\"]*)"$'),
                 'suicide': re.compile('^"([^\"]*)" committed suicide with "([^\"]*)"( \(world\))?$'),
                 'say': re.compile('^"([^\"]*)" say "(.*)"( \(dead\))?$'),
                 'sayTeam': re.compile('^"([^\"]*)" say_team "(.*)"( \(dead\))?$'),
                 'score': re.compile('^Team "([^\"]*)" scored "([^\"]*)" with "([^\"]*)" players$'),
                 'kick': re.compile('^Kick: "([^\"]*)" was kicked by "([^\"]*)" \(message "([^\"]*)"\)$'),
                 'secureKick': re.compile('^Secure: "([^\"]*)" was detected cheating and dropped from the server.$'),
                 'adminCommand1': re.compile('^\[ADMIN\] ADMIN Command: (.*) used command (.*)$'),
                 'adminCommand2': re.compile('^\[ADMIN\] \(say\) ADMIN Command: (.*) used command (.*)$'),
                 'adminSay': re.compile('^\[ADMIN\] \(([^\)]*)\) (.*)$'),
                 'adminKicked': re.compile('^\[ADMIN\] Kicked (.*)$'),
                 'hlguardFound': re.compile('^\[HLG[uU][aA][rR][dD]\] "([^\"]*)" (.*)$'),
                 'hlguardBanned': re.compile('^\[HLG[uU][aA][rR][dD]\] Executing banid on "([^\"]*)" for using cheats$')
                 }

    # adminSay needs to be checked last, so handle it this way.
    EVENT_RE_NAMES = EVENT_RES.keys()
    EVENT_RE_NAMES.remove('adminSay')
    EVENT_RE_NAMES.append('adminSay')

    PLAYER_RE = re.compile('^(.*)<([^\>]*)><([^\>]*)><([^\>]*)>$')

    def __init__(self, file=None):
        self.file = None
        self.now = None
        if file is not None:
            self.attach(file)

    def attach(self, file): self.file = file
    def detach(self): self.file = None

    def team(self, string):
        if not string:
            return None
        elif string[0] == 'T':
            return 'T'
        else:
            return 'CT'

    def won(self, local, won):
        if won and local >= 0 and won != 'STEAM_ID_PENDING':
            if won.find(':') >= 0:
                won = won.split(':', 1)[1]
            try:
                won = int(won)
                if type(won) is types.LongType:
                    raise ValueError, "2.3 support"
            except ValueError:
                won = 0
        else:
            won = 0
        return won

    def who(self, string):
        match = self.PLAYER_RE.search(string)
        if not match:
            return None
        name, local, won, team = match.groups()
        local = int(local)
        won = self.won(local, won)
        if team:
            team = self.team(team)
        else:
            team = None
        return name, local, won, team

    def address(self, string):
        seq = string.split(':', 1)
        seq[1] = int(seq[1])
        return tuple(seq)

    def line(self, line):
        line = line[:-1]
        if not line.startswith('L '):
            return ###
        line = line[2:]
        try:
            stamp, details = line.split(': ', 1)
            dateStr, timeStr = stamp.split(' - ', 1)
        except ValueError:
            return
        month, day, year = map(int, dateStr.split('/', 2))
        hour, minute, second = map(int, timeStr.split(':', 2))
        tuple = year, month, day, hour, minute, second, -1, -1, -1
        self.now = time.mktime(tuple)
        event = None
        for kind in self.EVENT_RE_NAMES:
            re = self.EVENT_RES[kind]
            match = re.search(details)
            if match is not None:
                if kind == 'logStart':
                    self.eventLogStart()
                elif kind == 'logEnd':
                    self.eventLogEnd()
                elif kind == 'loadingMap':
                    (name,) = match.groups()
                    self.eventLoadingMap(name)
                elif kind == 'startedMap':
                    name, crc = match.groups()
                    crc = int(crc)
                    self.eventStartedMap(name, crc)
                elif kind == 'serverSay':
                    (text,) = match.groups()
                    self.eventServerSay(text)
                elif kind == 'serverShutdown':
                    self.eventServerShutdown()
                elif kind == 'rcon':
                    number, password, command, address = match.groups()
                    number = long(number)
                    address = self.address(address)
                    self.eventRcon(number, password, command, address)
                elif kind == 'badRcon':
                    number, password, command, address = match.groups()
                    number = long(number)
                    address = self.address(address)
                    self.eventBadRcon(number, password, command, address)
                elif kind == 'cvarStart':
                    self.eventCvarStart()
                elif kind == 'cvarEnd':
                    self.eventCvarEnd()
                elif kind == 'cvar':
                    key, value = match.groups()
                    self.eventCvar(key, value)
                elif kind == 'trigger':
                    which, trigger = match.groups()
                    if trigger == 'Game_Commencing':
                        self.triggerGameCommencing()
                    elif trigger == 'Round_Start':
                        self.triggerRoundStart()
                    elif trigger == 'Round_End':
                        self.triggerRoundEnd()
                    elif trigger == 'Round_Draw':
                        self.triggerRoundDraw()
                    elif trigger.startswith('Restart_Round'):
                        self.triggerRoundRestarting()
                    elif trigger == 'Terrorists_Win':
                        self.triggerWin('T') ###
                    elif trigger == 'CTs_Win':
                        self.triggerWin('CT') ###
                    elif trigger == 'Begin_Bomb_Defuse_Without_Kit':
                        who = self.who(which)
                        self.triggerBeginDefuse(who, 0)
                    elif trigger == 'Begin_Bomb_Defuse_With_Kit':
                        who = self.who(which)
                        self.triggerBeginDefuse(who, 1)
                    elif trigger == 'Defused_The_Bomb':
                        who = self.who(which)
                        self.triggerDefusedBomb(who)
                    elif trigger == 'Bomb_Defused':
                        team = self.team(which)
                        self.triggerBombDefused(team)
                    elif trigger == 'Target_Saved':
                        team = self.team(which)
                        self.triggerTargetSaved(team)
                    elif trigger == 'Target_Bombed':
                        team = self.team(which)
                        self.triggerTargetBombed(team)
                    elif trigger == 'Spawned_With_The_Bomb':
                        who = self.who(which)
                        self.triggerSpawnedWithBomb(who)
                    elif trigger == 'Got_The_Bomb':
                        who = self.who(which)
                        self.triggerGotBomb(who)
                    elif trigger == 'Planted_The_Bomb':
                        who = self.who(which)
                        self.triggerPlantedBomb(who)
                    elif trigger == 'Touched_A_Hostage':
                        who = self.who(which)
                        self.triggerTouchedHostage(who)
                    elif trigger == 'Rescued_A_Hostage':
                        who = self.who(which)
                        self.triggerRescuedHostage(who)
                    elif trigger == 'Hostages_Not_Rescued':
                        team = self.team(which)
                        self.triggerHostagesNotRescued(team)
                    elif trigger == 'All_Hostages_Rescued':
                        team = self.team(which)
                        self.triggerAllHostagesRescued(team)
                    elif trigger == 'Killed_A_Hostage':
                        who = self.who(which)
                        self.triggerKilledHostage(who)
                    elif trigger == 'Dropped_The_Bomb':
                        who = self.who(which)
                        self.triggerDroppedBomb(who)
                    elif trigger == 'Became_VIP':
                        who = self.team(which)
                        self.triggerBecameVIP(who)
                    elif trigger == 'VIP_Assassinated':
                        team = self.team(which)
                        self.triggerVIPAssassinated(team)
                    elif trigger == 'VIP_Not_Escaped':
                        team = self.team(which)
                        self.triggerVIPNotEscaped(team)
                    elif trigger == 'VIP_Escaped':
                        team = self.team(which)
                        self.triggerVIPEscaped(team)
                    elif trigger == 'Escaped_As_VIP':
                        who = self.who(which)
                        self.triggerEscapedAsVIP(who)
                    elif trigger == 'Assassinated_The_VIP':
                        who = self.who(which)
                        self.triggerAssassinatedVIP(who)
                    else:
                        pass # by default, do nothing
                elif kind == 'connect':
                    who, address = match.groups()
                    who = self.who(who)
                    address = self.address(address)
                    self.eventConnect(who, address)
                elif kind == 'validated':
                    (who,) = match.groups()
                    who = self.who(who)
                    self.eventUserValidated(who)
                elif kind == 'disconnect':
                    (who,) = match.groups()
                    who = self.who(who)
                    self.eventDisconnect(who)
                elif kind == 'enter':
                    (who,) = match.groups()
                    who = self.who(who)
                    self.eventEnter(who)
                elif kind == 'join':
                    result = match.groups()
                    if len(result) == 2:
                        who, team = result
                        auto = 0
                    else:
                        who, team, dummy = result
                        auto = 1
                    who = self.who(who)
                    team = self.team(team)
                    self.eventJoin(who, team)
                elif kind == 'nameChange':
                    who, newName = match.groups()
                    who = self.who(who)
                    self.eventNameChange(who, newName)
                elif kind == 'attack1.3' or kind == 'attack1.4':
                    who, target, weapon, healthDamage, armorDamage, healthLeft, armorLeft = match.groups()
                    who = self.who(who)
                    target = self.who(target)
                    damage = int(healthDamage), int(armorDamage)
                    left = int(healthLeft), int(armorLeft)
                    self.eventAttack(who, target, weapon, damage, left)
                elif kind == 'kill':
                    who, target, weapon = match.groups()
                    who = self.who(who)
                    target = self.who(target)
                    self.eventKill(who, target, weapon)
                elif kind == 'suicide':
                    result = match.groups()
                    if len(result) == 2:
                        who, weapon = result
                        world = 0
                    else:
                        who, weapon, dummy = result
                        world = 1
                    who = self.who(who)
                    self.eventSuicide(who, weapon, world)
                elif kind == 'say' or kind == 'sayTeam':
                    result = match.groups()
                    if len(result) == 2:
                        who, what = result
                        dead = 0
                    else:
                        who, what, dummy = result
                        dead = 1
                    who = self.who(who)
                    args = (who, what)
                    if kind == 'say':
                        self.eventSay(*args)
                    else:
                        self.eventSayTeam(*args)
                elif kind == 'score':
                    team, score, playerCount = match.groups()
                    score = int(score)
                    playerCount = int(playerCount)
                    self.eventScore(team, score, playerCount)
                elif kind == 'kick':
                    who, what, message = match.groups()
                    who = self.who(who)
                    self.eventKick(who, what, message)
                elif kind == 'secureKick':
                    who, = match.groups()
                    who = self.who(who)
                    self.eventSecureKick(who)
                elif kind == 'adminSay':
                    type, text = match.groups()
                    if type == 'Admin':
                        type = 'chat'
                    self.adminSay(type, text)
                elif kind in ('adminCommand1', 'adminCommand2'):
                    name, what = match.groups()
                    verb, noun = what.split(' ', 1)
                    if not name:
                        name = '?'
                    self.adminCommand(name, verb, noun)
                elif kind == 'adminKicked':
                    (name,) = match.groups()
                    self.adminKicked(name)
                elif kind == 'hlguardFound':
                    who, what = match.groups()
                    who = self.who(who)
                    self.hlguardFound(who, what)
                elif kind == 'hlguardBanned':
                    (who,) = match.groups()
                    who = self.who(who)
                    self.hlguardBanned(who)
                else:
                    assert 0
                break
        else:
            self.eventUnknown(details)

    def go(self, file=None):
        if file is not None:
            self.attach(file)
        while 1:
            line = self.file.readline()
            if not line:
                break
            self.line(line)
        self.detach()

    def eventUnknown(self, details): raise NotImplementedError
    def eventLogStart(self): raise NotImplementedError
    def eventLogEnd(self): raise NotImplementedError
    def eventLoadingMap(self, map): raise NotImplementedError
    def eventStartedMap(self, map, crc): raise NotImplementedError
    def eventServerSay(self, text): raise NotImplementedError
    def eventServerShutdown(self): raise NotImplementedError
    def eventRcon(self, number, password, command, address): raise NotImplementedError
    def eventBadRcon(self, number, password, command, address): raise NotImplementedError
    def eventCvarStart(self): raise NotImplementedError
    def eventCvarEnd(self): raise NotImplementedError
    def eventCvar(self, key, value): raise NotImplementedError
    def eventConnect(self, who, address): raise NotImplementedError
    def eventUserValidated(self, who): raise NotImplementedError
    def eventDisconnect(self, who): raise NotImplementedError
    def eventEnter(self, who): raise NotImplementedError
    def eventJoin(self, who, team): raise NotImplementedError
    def eventNameChange(self, who, newName): raise NotImplementedError
    def eventAttack(self, who, target, weapon, damage, left): raise NotImplementedError
    def eventKill(self, who, target, weapon): raise NotImplementedError
    def eventSuicide(self, who, weapon, wasWorld): raise NotImplementedError
    def eventSay(self, who, what): raise NotImplementedError
    def eventSayTeam(self, who, what): raise NotImplementedError
    def eventScore(self, team, score, playerCount): raise NotImplementedError
    def eventKick(self, who, what, message): raise NotImplementedError
    def eventSecureKick(self, who): raise NotImplementedError

    def triggerGameCommencing(self): raise NotImplementedError
    def triggerRoundStart(self): raise NotImplementedError
    def triggerRoundEnd(self): raise NotImplementedError
    def triggerRoundDraw(self): raise NotImplementedError
    def triggerRoundRestarting(self): raise NotImplementedError
    def triggerWin(self, team): raise NotImplementedError
    def triggerBeginDefuse(self, who, hasKit): raise NotImplementedError
    def triggerDefusedBomb(self, who): raise NotImplementedError
    def triggerBombDefused(self, team): raise NotImplementedError
    def triggerTargetSaved(self, team): raise NotImplementedError
    def triggerTargetBombed(self, who): raise NotImplementedError
    def triggerSpawnedWithBomb(self, who): raise NotImplementedError
    def triggerGotBomb(self, who): raise NotImplementedError
    def triggerPlantedBomb(self, who): raise NotImplementedError
    def triggerTouchedHostage(self, who): raise NotImplementedError
    def triggerRescuedHostage(self, who): raise NotImplementedError
    def triggerHostagesNotRescued(self, who): raise NotImplementedError
    def triggerAllHostagesRescued(self, who): raise NotImplementedError
    def triggerKilledHostage(self, who): raise NotImplementedError
    def triggerDroppedBomb(self, who): raise NotImplementedError
    def triggerBecameVIP(self, who): raise NotImplementedError
    def triggerVIPAssassinated(self, team): raise NotImplementedError
    def triggerVIPNotEscaped(self, team): raise NotImplementedError
    def triggerVIPEscaped(self, team): raise NotImplementedError
    def triggerEscapedAsVIP(self, who): raise NotImplementedError
    def triggerAssassinatedVIP(self, who): raise NotImplementedError

    def adminSay(self, type, text): raise NotImplementedError
    def adminCommand(self, name, verb, noun): raise NotImplementedError
    def adminKicked(self, name): raise NotImplementedError

    def hlguardFound(self, who, what): raise NotImplementedError
    def hlguardBanned(self, who): raise NotImplementedError


class NullParser(Parser):

    """A parser subclass which simply implements a no-op for all its methods.
    This is useful if you plan to override only a few functions rather than
    most."""
    
    def eventUnknown(self, details): pass
    def eventLogStart(self): pass
    def eventLogEnd(self): pass
    def eventLoadingMap(self, map): pass
    def eventStartedMap(self, map, crc): pass
    def eventServerSay(self, text): pass
    def eventServerShutdown(self): pass
    def eventRcon(self, number, password, command, address): pass
    def eventBadRcon(self, number, password, command, address): pass
    def eventCvarStart(self): pass
    def eventCvarEnd(self): pass
    def eventCvar(self, key, value): pass
    def eventConnect(self, who, address): pass
    def eventUserValidated(self, who): pass
    def eventDisconnect(self, who): pass
    def eventEnter(self, who): pass
    def eventJoin(self, who, team): pass
    def eventNameChange(self, who, newName): pass
    def eventAttack(self, who, target, weapon, damage, left): pass
    def eventKill(self, who, target, weapon): pass
    def eventSuicide(self, who, weapon, wasWorld): pass
    def eventSay(self, who, what): pass
    def eventSayTeam(self, who, what): pass
    def eventScore(self, team, score, playerCount): pass
    def eventKick(self, who, what, message): pass
    def eventSecureKick(self, who): pass

    def triggerGameCommencing(self): pass
    def triggerRoundStart(self): pass
    def triggerRoundEnd(self): pass
    def triggerRoundDraw(self): pass
    def triggerRoundRestarting(self): pass
    def triggerWin(self, team): pass
    def triggerBeginDefuse(self, who, hasKit): pass
    def triggerDefusedBomb(self, who): pass
    def triggerBombDefused(self, team): pass
    def triggerTargetSaved(self, team): pass
    def triggerTargetBombed(self, who): pass
    def triggerSpawnedWithBomb(self, who): pass
    def triggerGotBomb(self, who): pass
    def triggerPlantedBomb(self, who): pass
    def triggerTouchedHostage(self, who): pass
    def triggerRescuedHostage(self, who): pass
    def triggerHostagesNotRescued(self, who): pass
    def triggerAllHostagesRescued(self, who): pass
    def triggerKilledHostage(self, who): pass
    def triggerDroppedBomb(self, who): pass
    def triggerBecameVIP(self, who): pass
    def triggerVIPAssassinated(self, team): pass
    def triggerVIPNotEscaped(self, team): pass
    def triggerVIPEscaped(self, team): pass
    def triggerEscapedAsVIP(self, who): pass
    def triggerAssassinatedVIP(self, who): pass

    def adminSay(self, type, text): pass
    def adminCommand(self, name, verb, noun): pass
    def adminKicked(self, name): pass

    def hlguardFound(self, who, what): pass
    def hlguardBanned(self, who): pass    


class SampleParser(Parser):
    def eventUnknown(self, details): print 'unknown: "%s"' % details
    def eventLogStart(self): print 'logStart'
    def eventLogEnd(self): print 'logEnd'
    def eventLoadingMap(self, map): print 'loadingMap: %s' % map
    def eventStartedMap(self, map, crc): print 'startedMap: %s (%d)' % (map, crc)
    def eventServerSay(self, text): print 'serverSay: "%s"' % text
    def eventServerShutdown(self): print 'serverShutdown'
    def eventRcon(self, number, password, command, address): print 'rcon: "%s" (%s)' % (command, address)
    def eventBadRcon(self, number, password, command, address): print 'badRcon: [%s] "%s" (%s)' % (password, command, address)
    def eventCvarStart(self): print 'cvarStart'
    def eventCvarEnd(self): print 'cvarEnd'
    def eventCvar(self, key, value): print 'cvar: %s to %s' % (key, value)
    def eventConnect(self, who, address): print 'connect: %s from %s' % (who, address)
    def eventUserValidated(self, who): print 'userValidated: %s' % (who,)
    def eventDisconnect(self, who): print 'disconnect: %s' % (who,)
    def eventEnter(self, who): print 'enter: %s' % (who,)
    def eventJoin(self, who, team): print 'join: %s joined %s' % (who, team)
    def eventNameChange(self, who, newName): print 'nameChange: %s changed name to %s' % (who, newName)
    def eventAttack(self, who, target, weapon, damage, now): print 'attack: %s attacked %s with %s %s/%s' % (who, target, weapon, damage, now)
    def eventKill(self, who, target, weapon): print 'kill: %s killed %s with %s' % (who, target, weapon)
    def eventSuicide(self, who, weapon, wasWorld): print 'suicide: %s killed self with %s%s' % (who, weapon, ['', ' with worldspawn'][wasWorld])
    def eventSay(self, who, what): print 'say: %s said "%s"' % (who, what)
    def eventSayTeam(self, who, what): print 'sayTeam: %s team-said "%s"' % (who, what)
    def eventScore(self, who, score, playerCount): print 'score: %s scored %s (with %s players)' % (who, score, playerCount)
    def eventKick(self, who, what, message): print 'kick: %s by %s, message "%s"' % (who, what, message)
    def eventSecureKick(self, who): print 'secureKick: %s' % (who,)

    def triggerGameCommencing(self): print 'gameCommencing'
    def triggerRoundStart(self): print 'roundStart'
    def triggerRoundEnd(self): print 'roundEnd'
    def triggerRoundDraw(self): print 'roundDraw'
    def triggerRoundRestarting(self): print 'roundRestarting'
    def triggerWin(self, team): print 'win: %s' % team
    def triggerBeginDefuse(self, who, hasKit): print 'beginDefuse: %s (%s)' % (who, ['without kit', 'with kit'][hasKit])
    def triggerDefusedBomb(self, who): print 'defusedBomb: %s' % who
    def triggerBombDefused(self, team): print 'bombDefused: %s' % team
    def triggerTargetBombed(self, who): print 'targetBombed: %s' % who
    def triggerTargetSaved(self, team): print 'targetSaved: %s' % team
    def triggerSpawnedWithBomb(self, who): print 'spawnedWithBomb: %s' % who
    def triggerGotBomb(self, who): print 'gotBomb: %s' % who
    def triggerPlantedBomb(self, who): print 'plantedBomb: %s' % who
    def triggerTouchedHostage(self, who): print 'touchedHostage: %s' % who
    def triggerRescuedHostage(self, who): print 'rescuedHostage: %s' % who
    def triggerHostagesNotRescued(self, team): print 'hostagesNotRescued: %s' % team
    def triggerAllHostagesRescued(self, team): print 'allHostagesRescued: %s' % team
    def triggerKilledHostage(self, who): print 'killedHostage: %s' % who
    def triggerDroppedBomb(self, who): print 'droppedBomb: %s' % who
    def triggerBecameVIP(self, who): print 'becameVIP: %s' % who
    def triggerVIPAssassinated(self, team): print 'vipAssassinated: %s' % team
    def triggerVIPNotEscaped(self, team): print 'vipNotEscaped: %s' % team
    def triggerVIPEscaped(self, team): print 'vipEscaped: %s' % team
    def triggerEscapedAsVIP(self, who): print 'escapedAsVIP: %s' % who
    def triggerAssassinatedVIP(self, who): print 'assassinatedVIP: %s' % who

    def adminSay(self, type, text): print 'adminSay: "%s" (%s)' % (text, type)
    def adminCommand(self, name, verb, noun): print 'adminCommand: %s executes %s %s' % (name, verb, noun)
    def adminKicked(self, name): print 'adminKicked: %s' % name

    def hlguardFound(self, who, what): print 'hlguardFound: %s had %s' % (who, what)
    def hlguardBanned(self, who): print 'hlguardBanned: %s' % (who,)


def main():
    import sys
    args = sys.argv[1:]
    if args:
        for filename in args:
            file = open(filename)
            parser = SampleParser(file)
            parser.go()
    else:
        file = sys.stdin
        parser = SampleParser(file)
        parser.go()

if __name__ == '__main__': main()
